package com.ltt.compass.weather.widget

import android.content.Context
import android.content.res.Resources
import android.graphics.Canvas
import android.graphics.Paint
import android.util.AttributeSet
import android.util.DisplayMetrics
import android.view.View
import android.view.animation.DecelerateInterpolator
import androidx.annotation.ColorInt
import androidx.core.content.ContextCompat
import androidx.viewpager2.widget.ViewPager2
import com.ltt.compass.R

class IndefinitePagerIndicator : View {

    private val DEFAULT_DOT_COUNT = 5
    private val DEFAULT_FADING_DOT_COUNT = 1
    private val DEFAULT_DOT_RADIUS_DP = 2
    private val DEFAULT_SELECTED_DOT_RADIUS_DP = 3f
    private val DEFAULT_DOT_SEPARATION_DISTANCE_DP = 10
    private fun dpToPx(dp: Float, resources: Resources): Int =(dp * ((resources.displayMetrics.densityDpi / DisplayMetrics.DENSITY_DEFAULT).toFloat())).toInt()

    constructor (context: Context) : super(context)

    constructor (context: Context, attrs: AttributeSet) : super(context, attrs) {
        initView(context, attrs)
    }


    private var viewPager: ViewPager2? = null
    private val interpolator = DecelerateInterpolator()
    private var dotCount = DEFAULT_DOT_COUNT
    private var fadingDotCount = DEFAULT_FADING_DOT_COUNT
    private var selectedDotRadiusPx = dpToPx(DEFAULT_SELECTED_DOT_RADIUS_DP, resources)
    private var dotRadiusPx = dpToPx(DEFAULT_DOT_RADIUS_DP.toFloat(), resources)
    private var dotSeparationDistancePx = dpToPx(DEFAULT_DOT_SEPARATION_DISTANCE_DP.toFloat(), resources)
    private var supportRtl = false
    private var verticalSupport = false

    @ColorInt
    private var dotColor: Int = ContextCompat.getColor(this.context, R.color.gray)
    @ColorInt
    private var selectedDotColor: Int = ContextCompat.getColor(this.context, R.color.white )
    private val selectedDotPaint = Paint()
    private val dotPaint = Paint()

    /**
     * The current pager position. Used to draw the selected dot if different size/color.
     */
    private var selectedItemPosition: Int = 0

    /**
     * A temporary value used to reflect changes/transition from one selected item to the next.
     */
    private var intermediateSelectedItemPosition: Int = 0

    /**
     * The scroll percentage of the viewpager or recyclerview.
     * Used for moving the dots/scaling the fading dots.
     */
    private var offsetPercent: Float = 0f

    private fun initView(context: Context, attrs: AttributeSet) {
        attrs.let {
            val typedArray = context.theme.obtainStyledAttributes(
                    attrs,
                    R.styleable.IndefinitePagerIndicator,
                    0,
                    0
            )
            dotCount = typedArray.getInteger(
                    R.styleable.IndefinitePagerIndicator_dotCount,
                    DEFAULT_DOT_COUNT
            )
            fadingDotCount = typedArray.getInt(
                    R.styleable.IndefinitePagerIndicator_fadingDotCount,
                    DEFAULT_FADING_DOT_COUNT
            )
            dotRadiusPx = typedArray.getDimensionPixelSize(
                    R.styleable.IndefinitePagerIndicator_dotRadius,
                    dotRadiusPx
            )
            selectedDotRadiusPx = typedArray.getDimensionPixelSize(
                    R.styleable.IndefinitePagerIndicator_selectedDotRadius,
                    selectedDotRadiusPx
            )
            dotColor = typedArray.getColor(R.styleable.IndefinitePagerIndicator_dotColor, dotColor)
            selectedDotColor = typedArray.getColor(
                    R.styleable.IndefinitePagerIndicator_selectedDotColor,
                    selectedDotColor
            )
            dotSeparationDistancePx = typedArray.getDimensionPixelSize(
                    R.styleable.IndefinitePagerIndicator_dotSeparation,
                    dotSeparationDistancePx
            )
            supportRtl =
                    typedArray.getBoolean(R.styleable.IndefinitePagerIndicator_supportRTL, false)
            verticalSupport = typedArray.getBoolean(
                    R.styleable.IndefinitePagerIndicator_verticalSupport,
                    false
            )
        }

        selectedDotPaint.apply {
            style = Paint.Style.FILL
            color = selectedDotColor
            isAntiAlias = true
        }
        dotPaint.apply {
            style = Paint.Style.FILL
            color = dotColor
            isAntiAlias = true
        }
    }

    /**
     * Iterate over the total pager item count and draw every dot based on position.
     *
     * Helper methods - getDotCoordinate(Int) & getRadius(Int)
     * will return values outside the calculated width or with an invalid radius
     * if the dot is not to be drawn.
     *
     * TODO: "Instagram style" drawing where all dots are drawn at once.
     */
    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        (0 until getPagerItemCount())
                .map { getDotCoordinate(it) }
                .forEach {
                    val xPosition: Float
                    val yPosition: Float
                    if (verticalSupport) {
                        xPosition = getDotYCoordinate().toFloat()
                        yPosition = height / 2 + it
                    } else {
                        xPosition = width / 2 + it
                        yPosition = getDotYCoordinate().toFloat()
                    }
                    canvas.drawCircle(xPosition, yPosition, getRadius(it), getPaint(it))
                }
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        val minimumViewSize = 2 * selectedDotRadiusPx
        if (verticalSupport) {
            setMeasuredDimension(minimumViewSize, getCalculatedWidth())
        } else {
            setMeasuredDimension(getCalculatedWidth(), minimumViewSize)
        }
    }

    private fun getDotCoordinate(pagerPosition: Int): Float =
            (pagerPosition - intermediateSelectedItemPosition) * getDistanceBetweenTheCenterOfTwoDots() +
                    (getDistanceBetweenTheCenterOfTwoDots() * offsetPercent)

    private fun getDotYCoordinate(): Int = selectedDotRadiusPx

    private fun getDistanceBetweenTheCenterOfTwoDots() = 2 * dotRadiusPx + dotSeparationDistancePx

    private fun getRadius(coordinate: Float): Float {
        val coordinateAbs = Math.abs(coordinate)
        val largeDotThreshold = dotCount.toFloat() / 2 * getDistanceBetweenTheCenterOfTwoDots()
        return when {
            coordinateAbs < getDistanceBetweenTheCenterOfTwoDots() / 2 -> selectedDotRadiusPx.toFloat()
            coordinateAbs <= largeDotThreshold -> dotRadiusPx.toFloat()
            else -> {
                val percentTowardsEdge = (coordinateAbs - largeDotThreshold) /
                        (getCalculatedWidth() / 2.01f - largeDotThreshold)
                interpolator.getInterpolation(1 - percentTowardsEdge) * dotRadiusPx
            }
        }
    }

    private fun getPaint(coordinate: Float): Paint = when {
        Math.abs(coordinate) < getDistanceBetweenTheCenterOfTwoDots() / 2 -> selectedDotPaint
        else -> dotPaint
    }

    private fun getCalculatedWidth(): Int {
        val maxNumVisibleDots = dotCount + 2 * fadingDotCount
        return (maxNumVisibleDots - 1) * getDistanceBetweenTheCenterOfTwoDots() + 2 * dotRadiusPx
    }


    fun attachToViewPager(viewPager: ViewPager2?) {
        this.viewPager = viewPager
        selectedItemPosition = viewPager?.currentItem!!
        postInvalidate()
    }

    private fun getPagerItemCount(): Int = when {
        viewPager != null -> viewPager?.adapter?.itemCount!!
        else -> 0
    }

    fun onPageScrolled(position: Int, positionOffset: Float) {
        intermediateSelectedItemPosition = position
        offsetPercent = positionOffset * -1
        invalidate()
    }

    fun onPageSelected(position: Int) {
        intermediateSelectedItemPosition = selectedItemPosition
        selectedItemPosition = position
        invalidate()
    }
}